package org.colllib.caches;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;
import java.util.HashMap;

/**
 * Cache to hold and access a read method for a given property name for
 * potentially different classes. 
 * <p>
 * This file is part of CollLib released under the terms of the LGPL V3.0.
 * See the file licenses/lgpl-3.0.txt for details.
 * 
 * @author marc.jackisch
 */
public class ReadMethodCache<V> {
   private final String              beanValue;
   private HashMap<Class<?>, Method> readMeths;

   public ReadMethodCache(String beanValue) {
      this.beanValue = beanValue;
   }

   public Method getReadMethod(Class<?> cl) {
      if(readMeths == null)
         readMeths = new HashMap<Class<?>, Method>();
      Method m = readMeths.get(cl);
      if(m != null)
         return m;
      try {
         BeanInfo bi = Introspector.getBeanInfo(cl);
         for(PropertyDescriptor p : bi.getPropertyDescriptors())
            if(p.getName().equals(beanValue) && p.getReadMethod() != null) {
               m = p.getReadMethod();
               readMeths.put(cl, m);
               return m;
            }
         throw new RuntimeException("No read method for property \""
            + beanValue + "\" found in class " + cl.getCanonicalName());
      }
      catch(Exception e) {
         throw new RuntimeException(e);
      }
   }

   public String getBeanValue() {
      return beanValue;
   }

   @SuppressWarnings("unchecked")
   public V invoke(Object t) {
      try {
         Object v = getReadMethod(t.getClass()).invoke(t);
         return (V) v;
      }
      catch(Exception e) {
         throw new RuntimeException(e);
      }
   }
}
