package org.colllib.caches;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;
import java.util.HashMap;

/**
 * Cache to hold and access a write method for a given property name for
 * potentially different classes. 
 * <p>
 * This file is part of CollLib released under the terms of the LGPL V3.0.
 * See the file licenses/lgpl-3.0.txt for details.
 * 
 * @author marc.jackisch
 */
public class WriteMethodCache<V> {
   private final String              beanValue;
   private HashMap<Class<?>, Method> writeMeths;

   public WriteMethodCache(String beanValue) {
      this.beanValue = beanValue;
   }

   public Method getWriteMethod(Class<?> cl) {
      if(writeMeths == null)
         writeMeths = new HashMap<Class<?>, Method>();
      Method m = writeMeths.get(cl);
      if(m != null)
         return m;
      try {
         BeanInfo bi = Introspector.getBeanInfo(cl);
         for(PropertyDescriptor p : bi.getPropertyDescriptors())
            if(p.getName().equals(beanValue) && p.getWriteMethod() != null) {
               m = p.getWriteMethod();
               writeMeths.put(cl, m);
               return m;
            }
         throw new RuntimeException("No write method for property \""
            + beanValue + "\" found in class " + cl.getCanonicalName());
      }
      catch(Exception e) {
         throw new RuntimeException(e);
      }
   }

   public String getBeanValue() {
      return beanValue;
   }

   public void invoke(Object t, Object v) {
      try {
         getWriteMethod(t.getClass()).invoke(t, new Object[] {v});
      }
      catch(Exception e) {
         throw new RuntimeException(e);
      }
   }
}

