package org.colllib.datastruct;

import java.io.Serializable;
import java.util.Iterator;

/**
 * This file is part of CollLib released under the terms of the LGPL V3.0. See
 * the file licenses/lgpl-3.0.txt for details.
 *
 * @author marc.jackisch
 */
public class IntRange implements Iterable<Integer>, Serializable {
   public int a, b;

   public IntRange(int a, int b) {
      this.a = Math.min(a, b);
      this.b = Math.max(a, b);
   }

   public boolean contains(Integer i) {
      return a <= i && i < b;
   }

   public IntRange intersection(IntRange r) {
      if(r.a < a)  return r.intersection(this);
      if(b < r.a)  return new IntRange(0, -1);
      if(r.b <= b) return r;
      return new IntRange(r.a, b);
   }

   public IntRange join(IntRange r) {
      return new IntRange(Math.min(a, r.a), Math.max(b, r.b));
   }

   public int[] toArray() {
      if(b <= a)
         return new int[0];
      int[] array = new int[b - a];
      for(int i = 0; i < array.length; i++)
         array[i] = a + i;
      return array;
   }

   @Override
   public String toString() {
      return "[" + a + ", " + b + "]";
   }

   @Override
   public Iterator<Integer> iterator() {
      return new Iterator<Integer>() {
         private int current = a;

         @Override
         public boolean hasNext() {
            return current < b;
         }

         @Override
         public Integer next() {
            return current++;
         }

         @Override
         public void remove() {
            throw new UnsupportedOperationException("Cannot remove a value from a number range!");
         }
      };
   }
}
