package org.colllib.datastruct;

import java.io.Serializable;
import java.util.Iterator;

/**
 * This file is part of CollLib released under the terms of the LGPL V3.0.
 * See the file licenses/lgpl-3.0.txt for details.
 *
 * @author marcel.billen
 */
public class LongRange implements Iterable<Long>, Serializable {
   public long a, b;

   public LongRange(long a, long b) {
      this.a = Math.min(a, b);
      this.b = Math.max(a, b);
   }

   public boolean contains(Long l) {
      return a <= l && l < b;
   }

   public LongRange intersection(LongRange r) {
      if(r.a < a)  return r.intersection(this);
      if(b < r.a)  return new LongRange(0L, -1L);
      if(r.b <= b) return r;
      return new LongRange(r.a, b);
   }

   public LongRange join(LongRange r) {
      return new LongRange(Math.min(a, r.a), Math.max(b, r.b));
   }

   public long[] toArray() {
      if(b <= a)
         return new long[0];
      long[] array = new long[(int) (b - a)];
      for(int l = 0; l < array.length; l++)
         array[l] = a + l;
      return array;
   }

   @Override
   public String toString() {
      return "[" + a + ", " + b + "]";
   }

   @Override
   public Iterator<Long> iterator() {
      return new Iterator<Long>() {
         private long current = a;

         @Override
         public boolean hasNext() {
            return current < b;
         }

         @Override
         public Long next() {
            return current++;
         }

         @Override
         public void remove() {
            throw new UnsupportedOperationException("Cannot remove a value from a number range!");
         }
      };
   }
}
