package org.colllib.generator;

import java.util.ArrayList;
import java.util.List;

import org.colllib.transformer.Transformer;
import org.colllib.transformer.TransformerCollection;
import org.colllib.util.CollectionUtil;

/**
 * This file is part of CollLib released under the terms of the LGPL V3.0.
 * See the file licenses/lgpl-3.0.txt for details.
 * 
 * @author marcel.billen
 */
public class GeneratorCollection {
   private GeneratorCollection() {
   }

   public static <S, D> Generator<S, D> el(Class<D> tClass, String accessName, List<String> expressions) {
      return new ElGenerator<S, D>(tClass, accessName, expressions);
   }
   
   private static class ElGenerator<S, D> implements Generator<S, D> {
      private List<Transformer<S, D>> transformers;

      private ElGenerator(Class<D> tClass, String accessName, List<String> expressions) {
         transformers = CollectionUtil.transform(
            expressions, 
            TransformerCollection.<String, Transformer<S, D>>constMethodCall(
               TransformerCollection.class, 
               null, 
               "el", 
               new Class<?>[] { Class.class, String.class, String.class }, 
               2, 
               new Object[] { tClass, accessName, null }));
      }

      @Override
      public Iterable<D> generate(S s) {
         ArrayList<D> r = new ArrayList<D>();
         for(Transformer<S, D> trans : transformers)
            r.add(trans.transform(s));
         return r;
      }
   }
   
   /**
    * Return a generator, which reads a property from a given element, which
    * must implement Iterable<B>.
    * 
    * E.g. consider a bean "Team", which contains a List<String> "members". The
    * generator can thus read the members list from a single team
    * via iterableProperty(Team t, "members") and be used to generate a
    * complete list of members for a collection of teams.
    */
   public static <A, B> Generator<A, B> iterableProperty(final String propName) {
      return new Generator<A, B>() {
         @Override
         public Iterable<B> generate(A s) {
            return TransformerCollection.<A, Iterable<B>>beanValue(propName).transform(s);
         }};
   }
}

