package org.colllib.util;

import java.io.Serializable;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import javax.el.ExpressionFactory;

import org.colllib.util.ellibs.CollectionLib;
import org.colllib.util.ellibs.StringLib;

import de.odysseus.el.util.SimpleContext;

/**
 * This file is part of CollLib released under the terms of the LGPL V3.0.
 * See the file licenses/lgpl-3.0.txt for details.
 * 
 * @author marc.jackisch
 */
public class ELAccessor implements Serializable {
   private static final String LIB_SUFFIX = "lib";

   private ExpressionFactory    factory;
   private SimpleContext        context;
   
   public ELAccessor() {
      this(StringLib.class, CollectionLib.class);
   }
   
   public ELAccessor(Class<?> ... libraryClasses) {
      factory = new de.odysseus.el.ExpressionFactoryImpl();
      context = new SimpleContext();
      
      for(Class<?> libraryClass : libraryClasses)
         addLibraryClass(context, libraryClass);
   }
   
   public void addLibraryClass(SimpleContext context, Class<?> libraryClass) {
      String basename = libraryClass.getSimpleName().toLowerCase();
      if(basename.endsWith(LIB_SUFFIX))
         basename = basename.substring(0, basename.length() - LIB_SUFFIX.length());
      for(Method func : libraryClass.getDeclaredMethods())
         if((func.getModifiers() & Modifier.STATIC) != 0)
            context.setFunction(basename, func.getName(), func);
   }
   
   public void setVariable(String name, Object v) {
      context.setVariable(name, factory.createValueExpression(v, v != null ? v.getClass() : Object.class));
   }
   
   public <R> R evaluateExpression(String exp, Class<R> resultClass) {
      return resultClass.cast(factory.createValueExpression(context, exp, resultClass).getValue(context));
   }
   
   public Object evaluateExpression(String exp) {
      return evaluateExpression(exp, Object.class);
   }
}
