package org.colllib.xml;

import java.util.AbstractList;
import java.util.List;

import org.colllib.filter.Filter;
import org.colllib.filter.FilterCollection;
import org.colllib.transformer.TransformerCollection;
import org.colllib.util.CollectionUtil;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Some utility functions to work with xml
 * <p>
 * This file is part of CollLib released under the terms of the LGPL V3.0.
 * See the file licenses/lgpl-3.0.txt for details.
 * 
 * @author marc.jackisch
 */
public class XMLUtil {
   private XMLUtil() {}
   
   /**
    * Convert a so-called NodeList into a list of nodes
    * 
    * @param nl the nodeList
    * @return the nodes as a list
    */
   public static List<Node> nodeList(final NodeList nl) {
      return new AbstractList<Node>() {
         @Override
         public Node get(int index) {
            return nl.item(index);
         }

         @Override
         public int size() {
            return nl.getLength();
         }
      };
   }
   
   /**
    * Fetch an iterator over the element type subnodes of a given element
    * 
    * @param root the base element 
    * @return an iterator for processing the direct child elements
    */
   public static Iterable<Element> childElementIterator(Element root) {
      return CollectionUtil.transform(
         CollectionUtil.applyFilter(
            nodeList(root.getChildNodes()),
            FilterCollection.isAssignableFromFilter(Element.class)),
         TransformerCollection.cast(Element.class));
   }
   
   public static Filter<Element> tagNameFilter(final String tagName) {
      return new Filter<Element>() {
         @Override
         public boolean matches(Element t) {
            return t.getTagName().equals(tagName);
         }
      };
   }
   
   public static Iterable<Element> childElementIterator(Element root, String tag) {
      return CollectionUtil.applyFilter(childElementIterator(root), tagNameFilter(tag));
   }
}

